
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace TheWizard
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        SpriteBatch mSpriteBatch;
        Wizard mWizard;
        TileEngine tEngine;
        Enemy mEnemy;
        LifeCounter mLives;
        Terrain mTerrain;

      

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            mSpriteBatch = new SpriteBatch(this.graphics.GraphicsDevice);
            ContentManager aLoader = new ContentManager(Services);
            mLives = new LifeCounter(this.graphics.GraphicsDevice, aLoader);
            mLives.setLives(3);
            tEngine = new TileEngine(this.graphics.GraphicsDevice, aLoader);
            mTerrain = new Terrain(this.graphics.GraphicsDevice, aLoader);
            mEnemy = new Enemy(this.graphics.GraphicsDevice, aLoader);
            mWizard = new Wizard(this.graphics.GraphicsDevice, aLoader, mTerrain);
            this.graphics.PreferredBackBufferHeight = 480;
            this.graphics.PreferredBackBufferWidth = 640;
            this.graphics.ApplyChanges();
            
            

            base.Initialize();
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            KeyboardState aKeyboard = Keyboard.GetState();
            if (aKeyboard.IsKeyDown(Keys.Escape) == true)
            {
                this.Exit();
            }//If Escape pressed

            mWizard.Update(gameTime.ElapsedGameTime.TotalSeconds);
            //mEnemy.Update(gameTime.ElapsedGameTime.TotalSeconds);

            if (Collision())
            {
                mLives.setLives(-1);
                mWizard.reset();
                mEnemy.reset();
            }
            if (GameOver())
            {
                Exit();
            }

            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);
            
            //tEngine.Draw(mSpriteBatch);
            mTerrain.Draw(mSpriteBatch);
            mEnemy.Draw(mSpriteBatch);
            mWizard.Draw(mSpriteBatch);
            mLives.Draw(mSpriteBatch);
           
            
            
            base.Draw(gameTime);

            
        }

        public Boolean Collision()
        {
            Sprite.Sprite obj1 = mWizard.getWizard();
            Sprite.Sprite obj2 = mEnemy.getEnemy();
            Boolean collide = false;

            if (obj2.Position.X < obj1.Position.X + obj1.Width)
            {
                if ((obj2.Position.Y  < obj1.Position.Y + obj1.Height) && (obj2.Position.Y > obj1.Position.Y))
                {
                    if (obj2.Position.X + obj2.Width < obj1.Position.X)
                    {
                        collide = false;
                    }
                    else
                    {
                        collide = true;
                    }
                }
            }//End if Obj2 X is less than Obj1X + Width
            return collide;
        }

        public Boolean GameOver()
        {
            int lives = mLives.getLives();

            if (lives == 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }
}