using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;
using TheWizard.Sprite;

using TheWizard;

namespace TheWizard
{
    class Wizard
    {
        //The texture for the Wizard and fireball
        Texture2D mWizardTexture;

        //The sprite to store positioning information about the Wizard
        Sprite.Sprite mWizard;
        
        bool mOnPlatform = false;
        Sprite.Sprite thePlatform;

        //The collection of sprites to store the positioning infomration about the Fireballs
        Dictionary<string, Sprite.Sprite> mFireballs = new Dictionary<string, global::TheWizard.Sprite.Sprite>();

        //dictionary of platforms
        Dictionary<string, Sprite.Sprite> mPlatforms = new Dictionary<string, global::TheWizard.Sprite.Sprite>();
        
        Gravity grav = new Gravity();
        
        //Constructor for the Wizard class

        public Wizard(GraphicsDevice theDevice, ContentManager theLoader, Terrain mTerrain)
        {
            //Load the image into the texture object
            mWizardTexture = theLoader.Load<Texture2D>("./Images/WizardSheet") as Texture2D;
            mWizard = new Sprite.Sprite(new Rectangle(20, 300, 50, 100), new Rectangle(0, 0, 90, 220));
            mWizard.Mass = 80;
            mWizard.Visible = true;
            mPlatforms = mTerrain.GetPlatforms();
        }//End Wizard(GraphicsDevice theDevice)

        //Update the Wizard and related objects
        public void Update(double elapsed)
        {
            UpdateWizard(elapsed);
            UpdateFireballs(elapsed);
            UpdateKeyboard(elapsed);
            
        }//End Update(float elapsed)

        //Draw the Wizard and related objects
        public void Draw(SpriteBatch theSpriteBatch)
        {
            theSpriteBatch.Begin(SpriteBlendMode.AlphaBlend);
            //Draw the Wizard
            theSpriteBatch.Draw(mWizardTexture, mWizard.Destination, mWizard.Source, Color.White);

            //Cycle through all of the Fireballs and draw all of the visible ones
            foreach (KeyValuePair<string, Sprite.Sprite> aFireball in mFireballs)
            {
                //If a visible fireball is found, then draw it
                if (aFireball.Value.Visible == true)
                {
                    theSpriteBatch.Draw(mWizardTexture, aFireball.Value.Destination, aFireball.Value.Source, Color.White);
                }//End if (aFireball.Value.Visible == true)
            }//Next (KeyValuePair<string, Sprite> aFireball in mFireballs)
            theSpriteBatch.End();
        }//End Draw(SpriteBatch theSpriteBatch)

        //The various states the Wizard can be in
        protected enum WizardState
        {
            Standing,
            Walking,
            Magicking,
            Ducking,
            Jumping
            
        }//End enum WizardState

        //The current state the Wizard is in (walking, jumping, etc.)
        protected WizardState mWizardState = WizardState.Standing;
        protected WizardState CurrentWizardState
        {
            get
            {
                return mWizardState;
            }
            set
            {
                mWizardState = value;
                switch (mWizardState)
                {
                    case WizardState.Standing:
                        {
                            //The Wizard is standing switch to the first frame of the walking wizard
                            mWizard.TexturePositionX = 0;
                            mWizard.TexturePositionY = 0;
                            mWizard.Velocity = new Vector2(0, mWizard.Velocity.Y);
                            
                            break;
                        }
                    case WizardState.Walking:
                        {
                            //The Wizard is walking switch to the first frame of the walking wizard
                            mWizard.TexturePositionX = 0;
                            mWizard.TexturePositionY = 0;
                            
                            break;
                        }
                    case WizardState.Ducking:
                        {
                            //The Wizard is ducking, switch to the wizard ducking frame
                            mWizard.TexturePositionX = 180;
                            mWizard.TexturePositionY = 220;
                            
                            break;
                        }
                    case WizardState.Magicking:
                        {
                            //The Wizard is using his magic. Swtich tot he wizard magicking frame and add a new fireball
                            mWizard.TexturePositionX = 180;
                            mWizard.TexturePositionY = 0;
                            
                            //Add a new fireball
                            AddFireball();
                            break;
                        }
                    
                }//End switch (mWizardState)
            }
        }//End CurrentWizardState Property

        //Update the Wizard
        protected void UpdateWizard(double elapsed)
        {
            //Decide which action needs updated based on the current state of the Wizard
            switch (CurrentWizardState)
            {
                //The Wizard is currently walking
                case WizardState.Walking:
                    {
                        UpdateWizardWalking(elapsed);
                        break;
                    }
                //the Wizard is currently jumping
                case WizardState.Jumping:
                    {
                        UpdateWizardJumping(elapsed);
                        break;
                    }
               
            }//End switch (CurrentWizardState)

            ApplyGravity();
           // mWizard.Position += mWizard.Velocity;
            CheckBorders();
           
            
        }//End UpdateWizard(float elapsed) 

        //Update the Wizard Walking
        private double mWizardHeadBobElapsed = 0; //Track the amount of time between the Wizard walking frames
        private void UpdateWizardWalking(double elapsed)
        {
           //If the wizard is currently walking then switch between his frames. The Wizard's head
            //bobs around as he shuffle along. To make sure this happens in a slower manner, an elapsed
            //time is used to determine whether it's time to switch to the new frame or not
            mWizardHeadBobElapsed += elapsed;

            //If enough time has passed, then switch to the next frame of the walking Wizard
            if (mWizardHeadBobElapsed > 0.2)
            {
                //Reset the elapsed time
                mWizardHeadBobElapsed = 0;

                //Switch between the two frames by setting the X position of the Texture start. 
                if (mWizard.TexturePositionX == 0)
                {
                    mWizard.TexturePositionX = 90;
                }
                else
                {
                    mWizard.TexturePositionX = 0;
                }//End if (mSprites.Sprites["Wizard"].TexturePositionX == 0)
            }//if (mWizardHeadBobElapsed > 0.2)

                       
            
        }//End UpdateWizardWalking(float elapsed)

        //Update the Wizard Jumping
        private double mWizardJumpElapsed = 0; //Track the amount of time between each phase of the Jump
        private Sprite.Sprite.eDirection mJumpDirection = Sprite.Sprite.eDirection.None; //Track the direction of the Jump (left, right or none)
        private void UpdateWizardJumping(double elapsed)
        {
            mWizardJumpElapsed += elapsed;

            //Pre-Jump. The Wizard crouches to prepare to jump
            if (mWizardJumpElapsed < .3)
            {
                //Change the Wizards texture to the crouching frame
                mWizard.TexturePositionX = 0;
                mWizard.TexturePositionY = 220;
            }
            //Jumping into the air (part 1)
            else if (mWizardJumpElapsed < .6)
            {
                //Change the Wizards texture to the arms in the air animation frame
                mWizard.TexturePositionX = 90;
               
            }
            
            //The Wizard is coming in for a landing*/
            else if (mWizardJumpElapsed < 1.5)
            {
                //If moving right, reduce moving right speed
                if (mWizard.Velocity.X > 0)
                {
                    mWizard.Velocity = new Vector2(1, mWizard.Velocity.Y);
                }
                //If moving left, reduce moving left speed
                else if (mWizard.Velocity.X < 0)
                {
                    mWizard.Velocity = new Vector2(-1, mWizard.Velocity.Y);
                }
                
                
            }
            //The Wizard has landed. Switch back to the crouching frame
            else if (mWizardJumpElapsed < 1.8)
            {
                //Change the Wizards texture to the crouching frame
                mWizard.TexturePositionX = 0;
                mWizard.TexturePositionY = 220;
                
            }
            //The Wizard has completed his jump. Switch the state back to Walking
            else
            {
                mWizardJumpElapsed = 0;
                mPreviousKeyPressed = Keys.None;
                CurrentWizardState = WizardState.Walking;
            }//End if (mWizardJumpElapsed < .3)
            
        }//End UpdateWizardJumping(float elapsed)

        private void ApplyGravity()
        {
            mWizard.Velocity = new Vector2(mWizard.Velocity.X, mWizard.Velocity.Y + grav.GetVelocity(mWizard));
            mWizard.Position += mWizard.Velocity;
        }

        
        //Move the Wizard
        private void CheckBorders()
        {

            
            if (mWizard.Position.X < 0)
            {
                mWizard.Position = new Vector2(0, mWizard.Position.Y);
            }//End if Wizard jump too far left
            else if (mWizard.Position.X > 640 - mWizard.Width)
            {
                mWizard.Position = new Vector2(640-mWizard.Width, mWizard.Position.Y);
            }

            if (mWizard.Position.Y + mWizard.Height > 300)
            {
                mWizard.Position = new Vector2(mWizard.Position.X, 300);
            }

            if (mWizard.Position.Y < 0)
            {
                mWizard.Position = new Vector2(mWizard.Position.X, 0);
            }
            
           
            
            
        }//End CheckBorders()

        public void OnPlatform()
        {
            foreach (KeyValuePair<string, Sprite.Sprite> aPlatform in mPlatforms)
            {
                //check if Wizard is on platform Y
                if ((mWizard.Position.Y + mWizard.Height) >= aPlatform.Value.Position.Y &&
                    (mWizard.Position.Y + mWizard.Height) <= (aPlatform.Value.Position.Y + aPlatform.Value.TextureHeight))
                {

                    //check if wizard is on platform X
                    if ((mWizard.Position.X + mWizard.Width) >= aPlatform.Value.Position.X &&
                        (mWizard.Position.X) <= (aPlatform.Value.Position.X + aPlatform.Value.TextureWidth))
                    {
                        mWizard.Position = new Vector2(mWizard.Position.X, aPlatform.Value.Position.Y - mWizard.Height + 10);
                        mOnPlatform = true;
                        
                    }

                }
            }
        }
        //Update the Fireballs 
        protected void UpdateFireballs(double elapsed)
        {
            //Put a spacer between the fireball shots so they don't come out on top of each other
            //and if the elapsed time has gone below 0, then reset it to 0.
            mFireballElapsed -= elapsed;
            if (mFireballElapsed < 0)
            {
                mFireballElapsed = 0;
            }//End if (mFireballElapsed < 0)

            //Flag used to determine if any fireballs are visible. If there isn't any visible
            //fireballs, then the Fireball elapsed timer needs to be reset so a new fireball
            //can be instantly added.
            bool aFireballVisible = false;

            //Cycle through all of the Fireballs to move them forward along the X-Axis
            foreach (KeyValuePair<string, Sprite.Sprite> aFireball in mFireballs)
            {
                //If the fireball is visible, then move it forward
                if (aFireball.Value.Visible == true)
                {
                    //If the fireball has moved far enough away from where it was fired (started), 
                    //then set it's visible property to false since it's went the length of it's range
                    if (aFireball.Value.Position.X > aFireball.Value.StartPosition.X + 400)
                    {
                        aFireball.Value.Visible = false;
                    }
                    //The fireball hasn't moved the length of it's range yet, so move it further forward
                    //along the X-Axis
                    else
                    {
                        //Move the Fireball forward
                        aFireball.Value.Position = new Vector2(aFireball.Value.Position.X + 5, aFireball.Value.Position.Y);

                        //Indicate that a visible fireball was found so there's 
                        //no need to reset the elapsed spacer for the fireballs
                        aFireballVisible = true;
                    }//End if (aFireball.Value.Position.X > aFireball.Value.StartPosition.X + 400)
                }//End if (aFireball.Value.Visible == false)
            }//Next (KeyValuePair<string, Sprite> aFireball in mFireballs)

            //If there weren't any visible fireballs, then reset the elapsed spacer timer
            //so that a new fireball can be fired instantly when appropriate
            if (aFireballVisible == false)
            {
                mFireballElapsed = 0;
            }//End if (aFireballVisible == false)

        }//End UpdateFireballs(float elapsed)

        //Update the keys being pressed and perform the appropriate actions corresponding to those keys
        Keys mPreviousKeyPressed = Keys.None; //Track the previous key that was pressed
        protected void UpdateKeyboard(double elapsed)
        {
            //Get the current state of the keyboard
            KeyboardState aKeyboard = Keyboard.GetState();

            //Check to see if the previous key pressed is still being pressed and that the Wizard isn't
            //currently jumping (no other action can be taken while the Wizard is jumping). If not, then reset the 
            //current state of the Wizard to "Standing" so taht a new state can be entered.
            if (aKeyboard.IsKeyDown(mPreviousKeyPressed) == false && CurrentWizardState != WizardState.Jumping)
            {
                mPreviousKeyPressed = Keys.None;
                CurrentWizardState = WizardState.Standing;

                //The Wizard is firing his magic fireballs
                if (aKeyboard.IsKeyDown(Keys.Space) == true)
                {
                    //Store the key being pressed and change the wizard state
                    mPreviousKeyPressed = Keys.Space;
                    mFireballElapsed = 0;
                    CurrentWizardState = WizardState.Magicking;
                }
                //The Wizard is jumping
                else if (aKeyboard.IsKeyDown(Keys.Up) == true)
                {
                    //Determine the jump direction (jumping straight up, to the left or to the right)
                    mJumpDirection = Sprite.Sprite.eDirection.None;
                    mWizard.Velocity = new Vector2(0, -40);
                    if (aKeyboard.IsKeyDown(Keys.Left) == true)
                    {
                        
                        mJumpDirection = Sprite.Sprite.eDirection.Left;
                    }
                    else if (aKeyboard.IsKeyDown(Keys.Right) == true)
                    {
                        

                        mJumpDirection = Sprite.Sprite.eDirection.Right;
                    }//if (aKeyboard.IsKeyDown(Keys.Left) == true)
                    
                    mPreviousKeyPressed = Keys.Up;
                    CurrentWizardState = WizardState.Jumping;
                }
                //The Wizard is ducking
                else if (aKeyboard.IsKeyDown(Keys.Down) == true)
                {
                    mPreviousKeyPressed = Keys.Down;
                    mWizard.Velocity = new Vector2(0, 0);
                    CurrentWizardState = WizardState.Ducking;
                }
                //The Wizard is walking to the left
                else if (aKeyboard.IsKeyDown(Keys.Left) == true)
                {
                    //Set the wizards direction to the left
                    mWizard.Direction = Sprite.Sprite.eDirection.Left;
                    mWizard.Velocity = new Vector2(-3, mWizard.Velocity.Y);
                    mPreviousKeyPressed = Keys.Left;
                    CurrentWizardState = WizardState.Walking;
                }
                //The Wizard is walking to the right
                else if (aKeyboard.IsKeyDown(Keys.Right) == true)
                {
                    //Set the wizards direction to the right
                    mWizard.Direction = Sprite.Sprite.eDirection.Right;
                    mWizard.Velocity = new Vector2(3, mWizard.Velocity.Y);
                    mPreviousKeyPressed = Keys.Right;
                    CurrentWizardState = WizardState.Walking;
                }//End if (aKeyboard.IsKeyDown(Keys.Space) == true)

            }
            else if (CurrentWizardState == WizardState.Magicking)
            {
                CurrentWizardState = WizardState.Magicking;
            }
            else if (CurrentWizardState != WizardState.Jumping && aKeyboard.IsKeyDown(Keys.Up) == true)
            {
                //Determine the jump direction (jumping straight up, to the left or to the right)
                mJumpDirection = Sprite.Sprite.eDirection.None;
                mWizard.Velocity = new Vector2(0, -40);
                
                if (aKeyboard.IsKeyDown(Keys.Left) == true)
                {
                    mWizard.Velocity = new Vector2(-2, mWizard.Velocity.Y);
                    mJumpDirection = Sprite.Sprite.eDirection.Left;
                }
                else if (aKeyboard.IsKeyDown(Keys.Right) == true)
                {
                    mWizard.Velocity = new Vector2(2, mWizard.Velocity.Y);
                    mJumpDirection = Sprite.Sprite.eDirection.Right;
                }//if (aKeyboard.IsKeyDown(Keys.Left) == true)

                mPreviousKeyPressed = Keys.Up;
                CurrentWizardState = WizardState.Jumping;
            }//End (aKeyboard.IsKeyDown(mPreviousKeyPressed) == false && CurrentWizardState != WizardState.Jumping)

            
        }//End UpdateKeyboard(float elapsed)

        //Track the time that has passed since the last fireball was created (helps with spacing the shots out)
        private double mFireballElapsed = 0;
        //Add a new fireball to the playing field
        protected void AddFireball()
        {
            //Only add a new fireball if enough time has elapsed since the last Fireball was shot
            if (mFireballElapsed == 0)
            {
                //Determine if a new fireball should be added, or if one already exists that we can just
                //reposition and turn visible again.
                bool aAddFireball = true;
                //Cycle through all of the Fireballs to see if their are any currently not visible that
                //can be reused rather than adding a new object
                foreach (KeyValuePair<string, Sprite.Sprite> aFireball in mFireballs)
                {
                    //If a Fireball is found in the dictionary object that isn't visible, then 
                    //that fireball will be reused as the new fireball being magicked.
                    if (aFireball.Value.Visible == false)
                    {
                        //Reposition the fireball at the Wizard current location (with a slight X offset so it 
                        //appear in front of the wizard) and make it visible.
                        aFireball.Value.StartPosition = new Vector2(mWizard.Position.X + 30, mWizard.Position.Y);
                        aFireball.Value.Position = new Vector2(mWizard.Position.X + 30, mWizard.Position.Y);
                        aFireball.Value.Visible = true;

                        //Reset the elapsed timer to space out the next fireball
                        mFireballElapsed = (float)0.5;

                        //Indicate the a new fireball does not need to be added
                        aAddFireball = false;

                        //Exit the for
                        break;
                    }//End if (aFireball.Value.Visible == false)
                }//Next (KeyValuePair<string, Sprite> aFireball in mFireballs)

                //If a new fireball needs to be added (all fireballs are currently visible or no fireballs 
                //are currently in the dictionary object), then add a new fireball, position it and make it visible
                if (aAddFireball == true)
                {
                    //Construct a key for the new Fireball, create the sprite and add it to the dictionary object
                    string aKey = "Fireball" + (mFireballs.Count + 1).ToString();
                    mFireballs.Add(aKey, new Sprite.Sprite(new Rectangle((int)mWizard.Position.X + 30,
                        (int)mWizard.Position.Y, 50, 100), new Rectangle(270, 0, 90, 220)));
                    mFireballs[aKey].Visible = true;

                    //Reset the elapsed time to space out the next fireball
                    mFireballElapsed = (float)0.5;
                }//End if (aAddFireball == true) 

            }//End if (mFireballElapsed == 0)
        }//End AddFireball(float elapsed)

        public Sprite.Sprite getWizard()
        {
            return mWizard;
        }

        public void reset()
        {
            mWizard.Position = new Vector2(40, mWizard.Position.Y);
        }

    }//End class Wizard
}
